/*
 * Copyright (C) 2003 BLADOX, s.r.o.  All rights reserved.
 *
 * This file is part of an example program for Turbo. This example
 * program may be used, distributed and modified without limitation.
 *
 */

#include <config.h>
#include <turbo/turbo.h>

#include <avr/sfr_defs.h>
#include <avr/io.h>

#include "i2cma.h"

void i2c_init (u8 myslaveaddress)
{
//__dbs("ii");//__dbs("\n");
//  outb (TWBR, 0x40);            // set TWI bit rate register to 64
//  outb (TWBR, 0x00);            // set TWI bit rate register to 0
//  outb (TWBR, 0x20);            // set TWI bit rate register to 32
//  outb (TWBR, 0x40);            // set TWI bit rate register to 64
//  outb (TWBR, 0x10);            // set TWI bit rate register to 16
  outb (TWBR, 0x10);		// set TWI bit rate register to 16
//  outb (TWBR, 0x00);          // set TWI bit rate register to 0
  outb (TWSR, 0x00);		// TWI prescaler = 0
  outb (TWAR, (myslaveaddress) << 1);	// set address
  outb (TWCR, 0x84);		// clear TWINT, enable TWI
}

// TWI send START condition
u8 i2c_send_start ()
{
  u8 status;

//__dbs("iss");__dbs("\n");
  delayUs (5);
  outb (TWCR, 0xA4);		// send START, clear TWINT
  while ((inb (TWCR) & 0x80) == 0);	//Wait for finishing of START
  status = (inb (TWSR) & 0xF8);	// return STATUS of TWI
//__dbs("status=0x");__dbch(status);__dbs("\n");
  return status;
}

// TWI send SLAVE ADDRESS with nWRITE bit = 0
u8 i2c_send_sla_w (u8 slaveaddress)
{
  u8 status;

//__dbs("issw");__dbs("\n");
  delayUs (5);
  outb (TWDR, ((slaveaddress << 1) | 0x00));	// send slave addres and nWRITE bit = 0
  outb (TWCR, 0x84);		// clear TWINT
  while ((inb (TWCR) & 0x80) == 0);	//Wait for finishing of SLA+W transmit
  status = (inb (TWSR) & 0xF8);	// return STATUS of TWI
//__dbs("status=0x");__dbch(status);__dbs("\n");
  return status;
}

// TWI send SLAVE ADDRESS with nWRITE bit = 1
u8 i2c_send_sla_r (u8 slaveaddress)
{
  u8 status;

//__dbs("issr");__dbs("\n");
  delayUs (5);
  outb (TWDR, ((slaveaddress << 1) | 0x01));	// send slave addres and nWRITE bit = 1
  outb (TWCR, 0x84);		// clear TWINT
  while ((inb (TWCR) & 0x80) == 0);	//Wait for finishing of SLA+W transmit
  status = (inb (TWSR) & 0xF8);	// return STATUS of TWI
//__dbs("status=0x");__dbch(status);__dbs("\n");
  return status;
}

// TWI send DATA
u8 i2c_send_data (u8 value)
{
  u8 status;

//__dbs("isd");__dbs("\n");
  delayUs (5);
  outb (TWDR, value);		// send DATA
  outb (TWCR, 0x84);		// clear TWINT
  while ((inb (TWCR) & 0x80) == 0);	//Wait for finishing of DATA transmit
  status = (inb (TWSR) & 0xF8);	// return STATUS of TWI
//__dbs("status=0x");__dbch(status);__dbs("\n");
  return status;
}

// TWI receive DATA
u8 i2c_receive_data (u8 * value, u8 lastFlag)
{
  u8 status;

//__dbs("ird");__dbs("\n");
  if (lastFlag == 0)
  {
//      sbi(TWCR,TWEA); //set ACK after receiving data
    outb (TWCR, 0x84 | (1 << TWEA));	// clear TWINT answer with ACK
  }
  else
  {
//      cbi(TWCR,TWEA); //clear ACK after receiving data
    outb (TWCR, 0x84);		// clear TWINT answer with NACK
  }
  while ((inb (TWCR) & 0x80) == 0);	//Wait for finishing of DATA transmit
  *value = inb (TWDR);		// read DATA
  status = (inb (TWSR) & 0xF8);	// return STATUS of TWI
//__dbs("status=0x");__dbch(status);__dbs("\n");
  return status;
}

// TWI send STOP condition
u8 i2c_send_stop ()
{
  u8 status;

//__dbs("issp");__dbs("\n");
  delayUs (5);
  outb (TWCR, 0x94);		// send STOP, clear TWINT
  status = (inb (TWSR) & 0xF8);	// return STATUS of TWI
//__dbs("status=0x");__dbch(status);__dbs("\n");
  return status;
}
